#ifdef GL_ES
precision highp float;
#endif

varying highp vec2 v_texcoord;

uniform vec2 u_resolution;
uniform float u_time;
uniform float u_slot1;
uniform float u_slot2;
uniform sampler2D u_tex0;
uniform float noiseFactor;
uniform float noiseDisplacement;
uniform vec2 uv_offset;
uniform float sampleScale;
uniform float flowSpeed;
uniform float u_saturation;
#define uTexture u_tex0

vec3 mod289(vec3 x){
    return x-floor(x*(1./289.))*289.;
}

vec2 mod289(vec2 x){
    return x-floor(x*(1./289.))*289.;
}

vec3 permute(vec3 x){
    return mod289(((x*34.)+1.)*x);
}

float snoise(vec2 v)
{
    const vec4 C=vec4(.211324865405187,// (3.0-sqrt(3.0))/6.0
        .366025403784439,// 0.5*(sqrt(3.0)-1.0)
        -.577350269189626,// -1.0 + 2.0 * C.x
        .024390243902439);// 1.0 / 41.0
    // First corner
    vec2 i=floor(v+dot(v,C.yy));
    vec2 x0=v-i+dot(i,C.xx);

    // Other corners
    vec2 i1;
    //i1.x = step( x0.y, x0.x ); // x0.x > x0.y ? 1.0 : 0.0
    //i1.y = 1.0 - i1.x;
    i1=(x0.x>x0.y)?vec2(1.,0.):vec2(0.,1.);
    // x0 = x0 - 0.0 + 0.0 * C.xx ;
    // x1 = x0 - i1 + 1.0 * C.xx ;
    // x2 = x0 - 1.0 + 2.0 * C.xx ;
    vec4 x12=x0.xyxy+C.xxzz;
    x12.xy-=i1;

    // Permutations
    i=mod289(i);// Avoid truncation effects in permutation
    vec3 p=permute(permute(i.y+vec3(0.,i1.y,1.))
        +i.x+vec3(0.,i1.x,1.));

    vec3 m=max(.5-vec3(dot(x0,x0),dot(x12.xy,x12.xy),dot(x12.zw,x12.zw)),0.);
    m=m*m;
    m=m*m;

    // Gradients: 41 points uniformly over a line, mapped onto a diamond.
    // The ring size 17*17 = 289 is close to a multiple of 41 (41*7 = 287)

    vec3 x=2.*fract(p*C.www)-1.;
    vec3 h=abs(x)-.5;
    vec3 ox=floor(x+.5);
    vec3 a0=x-ox;

    // Normalise gradients implicitly by scaling m
    // Approximation of: m *= inversesqrt( a0*a0 + h*h );
    m*=1.79284291400159-.85373472095314*(a0*a0+h*h);

    // Compute final noise value at P
    vec3 g;
    g.x=a0.x*x0.x+h.x*x0.y;
    g.yz=a0.yz*x12.xz+h.yz*x12.yw;
    return noiseFactor*dot(m,g);
}

float blendColorDodge(float base,float blend){
    return(blend==1.)?blend:min(base/(1.-blend),1.);
}

vec3 blendColorDodge(vec3 base,vec3 blend){
    return vec3(blendColorDodge(base.r,blend.r),blendColorDodge(base.g,blend.g),blendColorDodge(base.b,blend.b));
}

vec3 blendColorDodge(vec3 base,vec3 blend,float opacity){
    return(blendColorDodge(base,blend)*opacity+base*(1.-opacity));
}

vec3 brightnessContrast(in vec3 value,in float brightness,in float contrast)
{
    value=(value-.5)*contrast+.5+brightness;
    return value;
}

vec3 czm_saturation(vec3 rgb,float adjustment)
{
    const vec3 W=vec3(.2125,.7154,.0721);
    vec3 intensity=vec3(dot(rgb,W));
    return mix(intensity,rgb,adjustment);
}

float gradientNoise(in vec2 uv)
{
    const vec3 magic=vec3(.06711056,.00583715,52.9829189);
    return fract(magic.z*fract(dot(uv,magic.xy)));
}

void main(){
    vec2 st=gl_FragCoord.xy/u_resolution.xy;
    float s=snoise(vec2(st.x*1.,st.y*1.3+u_time*flowSpeed));

    // multiply the uv coord for 1 + the noise
    st*=vec2(1.+s*(noiseDisplacement*u_slot1));
    // apply page offset
    st.x+=(u_slot2*.5+.5);
    // Web 一个 0.5
    // st.x += ( 0.5 );
    st+=uv_offset;
    vec3 color=texture2D(uTexture,vec2(st.x*108./234.,st.y)/sampleScale).xyz;

    color.rgb=blendColorDodge(
        color.rgb,
        mix(vec3(0.,0.,0.),
        vec3(.14,0.,0.),
        0.)
    );

    // color.rgb = blendColorDodge(
    //     color.rgb,
    //     mix( vec3( 0.0, 0.0, 0.0 ),
    //          vec3(0.09, 0.08, 0.0),
    //          abs( sin( u_time ) )
    //     )
    // );

    color=czm_saturation(color,u_slot1*u_saturation);
    //brightness Effect
    color=brightnessContrast(color,-1.+u_slot1,1.);

    color+=color*(10.5/255.)*gradientNoise(gl_FragCoord.xy)-(10.5/255.);

    // vec2 uv = gl_FragCoord.xy/u_resolution.xy;
    // uv.x += ( u_slot2 * 0.5);
    // vec3 tex = texture2D(uTexture,vec2(uv.x,uv.y)/1.).xyz;
    gl_FragColor=vec4(color,1.);
}
